@@ -3,19 +3,23 @@
 #' Construction of prior distributions for extreme value model parameters
 #'
 #' Constructs a prior distribution for use as the argument \code{prior} in
-#' \code{\link{rpost}}.  The user can either specify their own prior function
+#' \code{\link{rpost}} and \code{\link{rpost_rcpp}}.
+#' The user can either specify their own prior function
+#' (using an R function or an external pointer to a compiled C++ function)
 #' and arguments for hyperparameters or choose from a list of in-built
 #' model-specific priors.  Note that the arguments \code{model = "gev"},
 #' \code{model = "pp"} and \code{model =="os"} are equivalent because
 #' a prior is specified is the GEV parameterisation in each of these cases.
 #' Note also that for \code{model = "pp"} the prior GEV parameterisation
 #' relates to the value of \code{noy} subsequently supplied to
-#' \code{\link{rpost}}.  The argument \code{model} is used for consistency
-#' with \code{rpost}.
+#' \code{\link{rpost}} or \code{\link{rpost_rcpp}}.
+#' The argument \code{model} is used for consistency with \code{rpost}.
 #'
 #' @param prior Either
 #' \itemize{
-#'   \item {A function that returns the value of the prior, or}
+#'   \item {An R function, or a pointer to a user-supplied compiled
+#'   C++ function, that returns the value of the prior
+#'   (see \strong{Examples}), or}
 #'   \item {A character string giving the name of the prior.
 #'     See \strong{Details} for a list of priors available for each model.}
 #' }
@@ -27,17 +31,25 @@
 #'   so that in the subsequent call to \code{rpost}, consistency of the
 #'   prior and extreme value model parameterisations can be checked.
 #' @param ... Further arguments to be passed to the user-supplied or
-#'   in-built prior function.  For details of the latter see \strong{Details}.
+#'   in-built prior function.  For details of the latter see \strong{Details}
+#'   and/or the relevant underlying function: \code{\link{gp_norm}},
+#'   \code{\link{gp_mdi}}, \code{\link{gp_flat}}, \code{\link{gp_flatflat}},
+#'   \code{\link{gp_jeffreys}}, \code{\link{gp_beta}},
+#'   \code{\link{gev_norm}}, \code{\link{gev_loglognorm}},
+#'   \code{\link{gev_mdi}}, \code{\link{gev_flat}}, \code{\link{gev_flatflat}},
+#'   \code{\link{gev_beta}}, \code{\link{gev_prob}}, \code{\link{gev_quant}}.
+#'   All these priors have the arguments \code{min_xi} (prior lower bound on
+#'   \eqn{\xi}) and \code{max_xi} (prior upper bound on \eqn{\xi}).
 #' @details Of the in-built named priors available in revdbayes only
-#'   those specified using \code{prior = "norm"} or
-#'   \code{prior = "loglognorm"} are proper.  Other proper
-#'   priors are available from the package evdbayes, via the
-#'   functions \code{\link[evdbayes]{prior.prob}},
+#'   those specified using \code{prior = "prob"} (\code{\link{gev_prob}}),
+#'   \code{prior = "quant"} (\code{\link{gev_quant}})
+#'   \code{prior = "norm"} (\code{\link{gev_norm}}) or
+#'   \code{prior = "loglognorm"} (\code{\link{gev_loglognorm}}) are proper.
+#'   If \code{model = "gev"} these priors are equivalent to priors available
+#'   in the evdbayes package, namely \code{\link[evdbayes]{prior.prob}},
 #'   \code{\link[evdbayes]{prior.quant}},
-#'   \code{\link[evdbayes]{prior.loglognorm}} and
-#'   \code{\link[evdbayes]{prior.norm}}.  The latter is equivalent to
-#'   using \code{model = "gev"} and \code{prior = "norm"} in
-#'   \code{set_prior}.
+#'   \code{\link[evdbayes]{prior.norm}} and
+#'   \code{\link[evdbayes]{prior.loglognorm}}.
 #'
 #'   The other in-built priors are improper, that is, the integral of the
 #'   prior function over its support is not finite.  Such priors do not
@@ -49,15 +61,26 @@
 #'   parameter of the GP distribution by \eqn{\sigma}, whereas we use
 #'   \eqn{\sigma_u} in the revdbayes vignette.
 #'
-#'   For all in-bulit priors the arguments \code{min_xi} and \code{max_xi} may
+#'   For all in-built priors the arguments \code{min_xi} and \code{max_xi} may
 #'   be supplied by the user.  The prior density is set to zero for any value
 #'   of the shape parameter \eqn{\xi} that is outside
 #'   (\code{min_xi}, \code{max_xi}).  This will override the default values
 #'   of \code{min_xi} and \code{max_xi} in the named priors detailed above.
 #'
-#'   \strong{Extreme value priors.} The names of the extreme value priors
-#'   set using \code{prior} and details of hyperparameters are:
+#'   \strong{Extreme value priors.} It is typical to use either
+#'   \code{prior = "prob"} (\code{\link{gev_prob}}) or
+#'   \code{prior = "quant"} (\code{\link{gev_quant}}) to set an informative
+#'   prior and one of the other prior (or a user-supplied function) otherwise.
+#'   The names of the in-built extreme value priors set using \code{prior}
+#'   and details of hyperparameters are:
 #' \itemize{
+#'   \item{\code{"prob"}.  A prior for GEV parameters \eqn{(\mu, \sigma, \xi)}
+#'     based on Crowder (1992).  See \code{\link{gev_prob}} for details.
+#'     See also Northrop et al. (2017) and Stephenson (2016).
+#'   }
+#'   \item{\code{"quant"}.  A prior for GEV parameters \eqn{(\mu, \sigma, \xi)}
+#'     based on Coles and Tawn (1996). See \code{\link{gev_quant}} for details.
+#'   }
 #'   \item {\code{"norm"}.
 #'
 #'   For \code{model = "gp"}:
@@ -152,8 +175,20 @@
 #'   input prior, i.e. either the name of the prior or a user-supplied
 #'   function.  The remaining components contain the numeric values of any
 #'   hyperparameters in the prior.
-#' @seealso \code{\link{rpost}} for sampling from an extreme value posterior
-#'   distribution.
+#' @seealso \code{\link{rpost}} and \code{\link{rpost_rcpp}} for sampling
+#'   from an extreme value posterior distribution.
+#' @seealso \code{\link{create_prior_xptr}} for creating an external
+#'   pointer to a C++ function to evaluate the log-prior density.
+#' @seealso \code{\link{rprior_prob}} and \code{\link{rprior_quant}} for
+#'   sampling from informative prior distributions for GEV parameters.
+#' @seealso \code{\link{gp_norm}}, \code{\link{gp_mdi}},
+#'   \code{\link{gp_flat}}, \code{\link{gp_flatflat}},
+#'   \code{\link{gp_jeffreys}}, \code{\link{gp_beta}} to see the arguments
+#'   for priors for GP parameters.
+#' @seealso \code{\link{gev_norm}}, \code{\link{gev_loglognorm}},
+#'   \code{\link{gev_mdi}}, \code{\link{gev_flat}}, \code{\link{gev_flatflat}},
+#'   \code{\link{gev_beta}}, \code{\link{gev_prob}}, \code{\link{gev_quant}}
+#'   to see the arguments for priors for GEV parameters.
 #' @seealso \code{\link[evdbayes]{prior.prob}},
 #'   \code{\link[evdbayes]{prior.quant}}, \code{\link[evdbayes]{prior.norm}}
 #'   and \code{\link[evdbayes]{prior.loglognorm}} for setting a prior
@@ -164,6 +199,13 @@
 #'   procedure for the generalized Pareto distribution.
 #'   \emph{Journal of Statistical Planning and Inference} \strong{137(2)},
 #'   473-483. \url{http://dx.doi.org/10.1016/j.jspi.2006.01.006}.
+#' @references Coles, S. G. and Tawn, J. A. (1996) A Bayesian analysis of
+#'   extreme rainfall data. \emph{Appl. Statist.}, \strong{45}, 463-478.
+#'   \url{http://dx.doi.org/10.2307/2986068}.
+#' @references Crowder, M. (1992) Bayesian priors based on parameter
+#'   transformation using the distribution function
+#'   \emph{Ann. Inst. Statist. Math.}, \strong{44}, 405-416.
+#'   \url{https://link.springer.com/article/10.1007/BF00050695}.
 #' @references Grimshaw, S. D. (1993) Computing Maximum Likelihood Estimates
 #'   for the Generalized Pareto Distribution.  \emph{Technometrics},
 #'   \strong{35(2)}, 185-191. \url{http://dx.doi.org/10.2307/1269663}.
@@ -171,7 +213,7 @@
 #' Quantile Estimation for the Generalized Pareto Distribution. Technometrics,
 #' 29(3), 339-349. \url{http://dx.doi.org/10.2307/1269343}.
 #' @references Martins, E. S. and Stedinger, J. R. (2000) Generalized maximum
-#'   likelihood generalized extreme value quantlie estimators for hydrologic
+#'   likelihood generalized extreme value quantile estimators for hydrologic
 #'   data, \emph{Water Resources Research}, \strong{36(3)}, 737-744.
 #'   \url{http://dx.doi.org/10.1029/1999WR900330}.
 #' @references Martins, E. S. and Stedinger, J. R. (2001) Generalized maximum
@@ -180,8 +222,18 @@
 #'   \url{http://dx.doi.org/10.1029/2001WR000367}.
 #' @references Northrop, P.J. and Attalides, N. (2016) Posterior propriety in
 #'   Bayesian extreme value analyses using reference priors
-#'   \emph{Statistica Sinica}, \strong{26(2)}, 721--743
+#'   \emph{Statistica Sinica}, \strong{26}(2), 721--743
 #'   \url{http://dx.doi.org/10.5705/ss.2014.034}.
+#' @references Northrop, P. J., Attalides, N. and Jonathan, P. (2017)
+#'   Cross-validatory extreme value threshold selection and uncertainty
+#'   with application to ocean storm severity.
+#'   \emph{Journal of the Royal Statistical Society Series C: Applied
+#'   Statistics}, \strong{66}(1), 93-120.
+#'   \url{http://dx.doi.org/10.1111/rssc.12159}
+#' @references Stephenson, A. (2016) Bayesian inference for extreme value
+#'   modelling.  In \emph{Extreme Value Modeling and Risk Analysis: Methods
+#'   and Applications} (eds D. K. Dey and J. Yan), 257-280, Chapman and Hall,
+#'   London. \url{http://dx.doi.org/10.1201/b19721-14}.
 #' @examples
 #' # Normal prior for GEV parameters (mu, log(sigma), xi).
 #' mat <- diag(c(10000, 10000, 100))
@@ -201,9 +253,14 @@
 #'          (ab[2] - 1) * log(1 - x[2]))
 #' }
 #' up <- set_prior(prior = u_prior_fn, ab = c(2, 2))
+#'
+#' # A user-defined prior using a pointer to a C++ function
+#' ptr_gp_flat <- create_prior_xptr("gp_flat")
+#' u_prior_ptr <- set_prior(prior = ptr_gp_flat, model = "gp")
 #' @export
 set_prior <- function(prior = c("norm", "loglognorm", "mdi", "flat",
-                                "flatflat", "jeffreys", "beta"),
+                                "flatflat", "jeffreys", "beta", "prob",
+                                "quant"),
                       model = c("gev", "gp", "pp", "os"), ...) {
   if (length(model) > 1) {
     warning("model not supplied: model == \"gev\" has been used.",
@@ -225,6 +282,13 @@ set_prior <- function(prior = c("norm", "loglognorm", "mdi", "flat",
     temp$prior <- new_prior
     return(structure(temp, class = "evprior", model = model))
   }
+  # If prior is a pointer to an external C++ function then just return it in
+  # the required format.  We don't need to worry about adding the argument
+  # trendsd because we don't need the prior to work with the evdbayes package.
+  if (class(prior) == "externalptr") {
+    temp <- list(prior = prior, ...)
+    return(structure(temp, class = "evprior", model = model))
+  }
   # Otherwise, call the appropriate function to set the prior with name prior.
   prior <- match.arg(prior)
   temp <- switch(model, gp = gp_prior(prior, ...), gev = gev_prior(prior, ...),
@@ -238,6 +302,33 @@ gp_prior <- function(prior = c("norm", "mdi", "flat", "flatflat", "jeffreys",
                                "beta"), ...) {
   prior <- match.arg(prior)
   temp <- list(prior = paste("gp_", prior, sep=""), ...)
+  # For v1.2.0 ...
+  # Add default value for min_xi in Jeffreys prior if it isn't present.
+  if (prior == "jeffreys" & is.null(temp$min_xi)) {
+    temp$min_xi = -1/2
+  }
+  # Add default value for min_xi in the MDI prior if it isn't present.
+  if (prior == "mdi" & is.null(temp$min_xi)) {
+    temp$min_xi = -1
+  }
+  # ... and similarly for beta prior.
+  if (prior == "beta" & is.null(temp$min_xi)) {
+    temp$min_xi = -1/2
+  }
+  if (prior == "beta" & is.null(temp$max_xi)) {
+    temp$max_xi = 1/2
+  }
+  # Force min_xi and max_xi to be present in the list temp.
+  temp$min_xi <- max(temp$min_xi, -Inf)
+  temp$max_xi <- min(temp$max_xi, Inf)
+  # ... add default value for a in MDI prior if it isn't present.
+  if (prior == "mdi" & is.null(temp$a)) {
+    temp$a <- 1
+  }
+  # ... add default value for pq in beta-type prior if it isn't present.
+  if (prior == "beta" & is.null(temp$pq)) {
+    temp$pq = c(6, 9)
+  }
   # Check for unused hyperparameter names and drop them
   hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a",
                      flat = NULL, jeffreys = NULL, beta = "pq")
@@ -295,6 +386,21 @@ gp_prior <- function(prior = c("norm", "mdi", "flat", "flatflat", "jeffreys",
 
 # ------------------------------ specific GP priors -------------------------- #
 
+#' Bivariate normal prior for GP parameters (\eqn{log \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 2.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param mean A numeric vector of length 2.  Prior mean.
+#' @param icov A 2x2 numeric matrix.
+#'   The inverse of the prior covariance matrix.
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gp_norm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
                     trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
@@ -302,11 +408,32 @@ gp_norm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
   }
   pars[1] <- log(pars[1])
   cpar <- pars - mean
-  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2]
-            + icov[3] * cpar[2] ^ 2
+  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2] +
+    icov[3] * cpar[2] ^ 2
   return(-ld / 2 - pars[1])
 }
 
+#' Maximal data information (MDI) prior for GP parameters
+#' (\eqn{\sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param a A numeric scalar.  The default value, Euler's constant, gives the
+#'   MDI prior.
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#'   See Northrop and Attalides (2016) for details.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @references Northrop, P.J. and Attalides, N. (2016) Posterior propriety in
+#'   Bayesian extreme value analyses using reference priors
+#'   \emph{Statistica Sinica}, \strong{26(2)}, 721--743
+#'   \url{http://dx.doi.org/10.5705/ss.2014.034}.
+#' @export
 gp_mdi <- function(pars, a = 1, min_xi = -1, max_xi = Inf, trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
     return(-Inf)
@@ -314,6 +441,19 @@ gp_mdi <- function(pars, a = 1, min_xi = -1, max_xi = Inf, trendsd = 0) {
   return(-log(pars[1]) - a * pars[2])
 }
 
+#' Flat prior for GP parameters (\eqn{log \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 2.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gp_flat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
     return(-Inf)
@@ -321,6 +461,19 @@ gp_flat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   return(-log(pars[1]))
 }
 
+#' Flat prior for GP parameters (\eqn{\sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 2.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gp_flatflat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
     return(-Inf)
@@ -328,6 +481,19 @@ gp_flatflat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   return(1)
 }
 
+#' Jeffreys prior for GP parameters (\eqn{\sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 2.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gp_jeffreys <- function(pars, min_xi = -1/2, max_xi = Inf, trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
     return(-Inf)
@@ -335,6 +501,20 @@ gp_jeffreys <- function(pars, min_xi = -1/2, max_xi = Inf, trendsd = 0) {
   return(-log(pars[1]) - log(1 + pars[2]) - log(1 + 2 * pars[2]) / 2)
 }
 
+#' Beta-type prior for GP shape parameter \eqn{\xi}
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 2.
+#'   GP parameters (\eqn{\sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param pq A numeric vector of length 2.
+#'   See \code{\link{set_prior}} for details.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gp_beta <- function(pars, min_xi = -1 / 2, max_xi = 1 / 2, pq = c(6, 9),
                     trendsd = 0) {
   if (pars[1] <= 0 | pars[2] < min_xi | pars[2] > max_xi) {
@@ -347,12 +527,38 @@ gp_beta <- function(pars, min_xi = -1 / 2, max_xi = 1 / 2, pq = c(6, 9),
 # ================================= GEV priors =================================
 
 gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
-                              "beta"), ...) {
+                              "beta", "prob", "quant"), ...) {
   prior <- match.arg(prior)
   temp <- list(prior = paste("gev_", prior, sep=""), ...)
+  # For v1.2.0 ...
+  # Add default value for min_xi in the MDI prior if it isn't present.
+  if (prior == "mdi" & is.null(temp$min_xi)) {
+    temp$min_xi = -1
+  }
+  # Add default values for min_xi and max_xi in the beta prior if they
+  # aren't present.
+  if (prior == "beta" & is.null(temp$min_xi)) {
+    temp$min_xi = -1/2
+  }
+  if (prior == "beta" & is.null(temp$max_xi)) {
+    temp$max_xi = 1/2
+  }
+  # Force min_xi and max_xi to be present in the list temp.
+  temp$min_xi <- max(temp$min_xi, -Inf)
+  temp$max_xi <- min(temp$max_xi, Inf)
+  # ... add default value for a in MDI prior if it isn't present.
+  if (prior == "mdi" & is.null(temp$a)) {
+    temp$a <- 0.5772156649015323
+  }
+  # ... add default value for pq in beta-type prior if it isn't present.
+  if (prior == "beta" & is.null(temp$pq)) {
+    temp$pq = c(6, 9)
+  }
   # Check for unused hyperparameter names and drop them
-  hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a",
-                      flat = NULL, beta = "pq")
+  hpar_vec <- switch(prior, norm = c("mean", "cov"),
+                     loglognorm = c("mean", "cov"), mdi = "a", flat = NULL,
+                     beta = "pq", prob = c("quant", "alpha"),
+                     quant = c("prob", "shape", "scale"))
   hpar_vec <- c(hpar_vec, "min_xi", "max_xi")
   temp <- hpar_drop(temp, hpar_vec)
   # Check for problems with min_xi and/or max_xi
@@ -367,7 +573,7 @@ gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
     }
   }
   # Check admissibility of hyperparameters
-  if (prior == "norm") {
+  if (prior == "norm" | prior == "loglognorm") {
     mean <- temp$mean
     cov <- temp$cov
     if (length(mean) != 3 | mode(mean) != "numeric")
@@ -394,6 +600,44 @@ gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
     if (length(pq) != 2 | !is.numeric(pq) | any(pq <= 0) )
         stop("pq must be a non-negative numeric vector of length 2")
   }
+  if (prior == "prob") {
+    if (is.null(temp$quant)) {
+      stop("quant must be supplied when prior = prob")
+    }
+    if (is.null(temp$alpha)) {
+      stop("alpha must be supplied when prior = prob")
+    }
+    if (length(temp$quant) != 3 | mode(temp$quant) != "numeric") {
+      stop("quant must be a numeric vector of length three")
+    }
+    if (length(temp$alpha) != 4 | mode(temp$alpha) != "numeric") {
+      stop("alpha must be a numeric vector of length four")
+    }
+    # Sort quant to ensure that the quantiles are increasing.
+    temp$quant <- sort(temp$quant, decreasing = FALSE)
+  }
+  if (prior == "quant") {
+    if (is.null(temp$prob)) {
+      stop("prob must be supplied when prior = quant")
+    }
+    if (is.null(temp$shape)) {
+      stop("shape must be supplied when prior = quant")
+    }
+    if (is.null(temp$scale)) {
+      stop("scale must be supplied when prior = quant")
+    }
+    if (length(temp$prob) != 3 | mode(temp$prob) != "numeric") {
+      stop("prob must be a numeric vector of length three")
+    }
+    if (length(temp$shape) != 3 | mode(temp$shape) != "numeric") {
+      stop("shape must be a numeric vector of length three")
+    }
+    if (length(temp$scale) != 3 | mode(temp$scale) != "numeric") {
+      stop("scale must be a numeric vector of length three")
+    }
+    # Sort prob to ensure that the corresponding quantiles are increasing.
+    temp$prob <- sort(temp$prob, decreasing = TRUE)
+  }
   # Add trendsd to the prior list so that the prior will work with the
   # evdbayes function posterior().
   temp$trendsd <- 0
@@ -402,6 +646,21 @@ gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
 
 # ------------------------------ specific GEV priors ------------------------- #
 
+#' Trivariate normal prior for GEV parameters (\eqn{\mu, log \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param mean A numeric vector of length 3.  Prior mean.
+#' @param icov A 3x3 numeric matrix.
+#'   The inverse of the prior covariance matrix.
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gev_norm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
                      trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
@@ -409,34 +668,78 @@ gev_norm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
   }
   pars[2] <- log(pars[2])
   cpar <- pars - mean
-  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2]
-            + 2 * icov[3] * cpar[1] * cpar[3] + icov[4] * cpar[2] ^ 2
-            + 2 * icov[5] * cpar[2] * cpar[3] + icov[6] * cpar[3] ^ 2
+  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2] +
+    2 * icov[3] * cpar[1] * cpar[3] + icov[4] * cpar[2] ^ 2 +
+    2 * icov[5] * cpar[2] * cpar[3] + icov[6] * cpar[3] ^ 2
   return(-ld / 2 - pars[2])
 }
 
+#' Trivariate normal prior for GEV parameters (\eqn{log \mu, log \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param mean A numeric vector of length 3.  Prior mean.
+#' @param icov A 3x3 numeric matrix.
+#'   The inverse of the prior covariance matrix.
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gev_loglognorm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
                            trendsd = 0) {
-  if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
+  if (pars[1] <= 0 | pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
   }
+  pars[1] <- log(pars[1])
   pars[2] <- log(pars[2])
-  pars[3] <- log(pars[3])
   cpar <- pars - mean
-  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2]
-            + 2 * icov[3] * cpar[1] * cpar[3] + icov[4] * cpar[2] ^ 2
-            + 2 * icov[5] * cpar[2] * cpar[3] + icov[6] * cpar[3] ^ 2
-  return(-ld / 2 - pars[2] - pars[3])
+  ld <- icov[1] * cpar[1] ^ 2 + 2 * icov[2] * cpar[1] * cpar[2] +
+    2 * icov[3] * cpar[1] * cpar[3] + icov[4] * cpar[2] ^ 2 +
+    2 * icov[5] * cpar[2] * cpar[3] + icov[6] * cpar[3] ^ 2
+  return(-ld / 2 - pars[2] - pars[1])
 }
 
+#' Maximal data information (MDI) prior for GEV parameters
+#' (\eqn{\mu, \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param a A numeric scalar.  The default value, Euler's constant, gives the
+#'   MDI prior.
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gev_mdi <- function(pars, a=0.5772156649015323, min_xi=-1, max_xi=Inf,
                     trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
   }
-  return(-log(pars[1]) - a * pars[3])
+  return(-log(pars[2]) - a * pars[3])
 }
 
+#' Flat prior for GEV parameters (\eqn{\mu, log \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gev_flat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
@@ -444,6 +747,19 @@ gev_flat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   return(-log(pars[2]))
 }
 
+#' Flat prior for GEV parameters (\eqn{\mu, \sigma, \xi})
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#'   Must not be \code{-Inf} because this results in an improper posterior.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
 gev_flatflat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
@@ -451,12 +767,27 @@ gev_flatflat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
   return(1)
 }
 
-gev_beta <- function(pars, min_xi = -1 / 2, max_xi = 1 / 2, pq = c(6, 9)) {
+#' Beta-type prior for GEV shape parameter \eqn{\xi}
+#'
+#' For information about this and other priors see \code{\link{set_prior}}.
+#'
+#' @param pars A numeric vector of length 3.
+#'   GEV parameters (\eqn{\mu, \sigma, \xi}).
+#' @param min_xi  A numeric scalar.  Prior lower bound on \eqn{\xi}.
+#' @param max_xi  A numeric scalar.  Prior upper bound on \eqn{\xi}.
+#' @param pq A numeric vector of length 2.
+#'   See \code{\link{set_prior}} for details.
+#' @param trendsd  Has no function other than to achieve compatability with
+#'   function in the evdbayes package.
+#' @return The log of the prior density.
+#' @export
+gev_beta <- function(pars, min_xi = -1 / 2, max_xi = 1 / 2, pq = c(6, 9),
+                     trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
   }
-  return(-log(pars[2]) + (pq[1] - 1) * log(1 / 2 + pars[3]) +
-           (pq[2] - 1) * log(1 / 2 - pars[3]))
+  return(-log(pars[2]) + (pq[1] - 1) * log(pars[3] - min_xi) +
+           (pq[2] - 1) * log(max_xi - pars[3]))
 }
 
 # ================================ hpar_drop ===============================
